#include	"dmonitor.h"
#include	"init.h"

void	rig_write (int length, char buff[]);
int	getFifo (char pkt[]);
void	LastFrameSend (void);
void	rig_led_onoff(void);
void	inet_led_onoff(void);
void	buff_hold_size_write (int n);
void	dummy_last_frame (void);
void	jitter_putFifo (void);
int	send_dest_keep_alive(void);

extern	char	dummy_frame_1[];
extern	char	dummy_frame_2[];
extern	char	dummy_frame_3[];

int	icom_open(void)
{
        struct termios attr;
	int	wrt_len;

        rig_fd = open(RIG_DEVICE, O_RDWR | O_NOCTTY | O_NONBLOCK);
        if (rig_fd < 0) 
	{
		syslog (LOG_ERR, "USB open error on ICOM rig");
                return FALSE;
        }

        tcgetattr(rig_fd, &attr);
        save_attr = attr;

	bzero (&attr, sizeof(attr));
        cfsetispeed(&attr, IDxxPlus_SPEED);
        cfsetospeed(&attr, IDxxPlus_SPEED);
        cfmakeraw(&attr);
        attr.c_cflag |= CS8 | CLOCAL | CREAD;
        attr.c_iflag = IGNPAR;
        attr.c_oflag = 0;
        attr.c_lflag = 0;
        attr.c_cc[VMIN] = 0;
        attr.c_cc[VTIME] = 0;

        tcsetattr(rig_fd, TCSANOW, &attr);
	ioctl (rig_fd, TCSETS, &attr);

	if (!scan_sw) syslog (LOG_INFO, "RIG(ID-xxPlus) open");
	else fprintf (status_fd, "RIG(ID-xxPlus) open.\n");
	FD_SET (rig_fd, &fd_save);
	return TRUE;
}

void	rig_close (void)
{
	int	ret;

	tcsetattr (rig_fd, TCSANOW, &save_attr);
	ret = close (rig_fd);
	FD_CLR (rig_fd, &fd_save);
	rig_fd = 0;
	if (rig_type == ICOM)
	{
		if(!scan_sw) syslog (LOG_INFO, "RIG(ICOM) close.");
		else fprintf (status_fd, "RIG(ICOM) close.\n");
	}
	else if (rig_type == DVAP)
	{
		if(!scan_sw) syslog (LOG_INFO, "DVAP close.");
		else fprintf (status_fd, "DVAP close.\n");
	}
}

void    pselectSet(void)
{
        FD_ZERO (&fd_save);
        timeout.tv_sec = 0;
        timeout.tv_nsec = 10000000;
}

void    handler (int sig)
{
	sig_term = FALSE;
        if (sig == SIGINT)  sig_term = TRUE;
	else if (sig == SIGUSR1)
	{
		fifo_new_hold += 50;
		syslog (LOG_INFO, "New FiFo buffer hold size %d", fifo_new_hold);
		buff_hold_size_write(fifo_new_hold);
	}
        else if (sig == SIGUSR2)
        {
		fifo_new_hold -= 50;
		if (fifo_new_hold < 20) fifo_new_hold = 20;
		syslog (LOG_INFO, "New FiFo buffer hold size %d", fifo_new_hold);
		buff_hold_size_write (fifo_new_hold);
        }
	else
		syslog (LOG_INFO, "signal %d", sig);
}

void	buff_hold_size_write (int cnt)
{
	FILE *cnt_file;
        struct  stat    sb;

        if (stat ("/var/www/", &sb)) return;
        if ((sb.st_mode & S_IFMT) != S_IFDIR) return;

	cnt_file = fopen (BUFF_HOLD, "w");
	fprintf (cnt_file, "%d\n", cnt);
	fclose (cnt_file);
}

int	buff_hold_size_read (void)
{
	FILE *cnt_file;
	struct	stat	sb;
	int	cnt;
	char	str[10];

	if (stat (BUFF_HOLD, &sb)) return 20;
	if ((sb.st_mode & S_IFMT) != S_IFREG)
	{
		buff_hold_size_write (20);
		return 20;
	}
	cnt_file = fopen (BUFF_HOLD, "r");
	if (fgets (str, 10, cnt_file))
	{
		cnt = atoi(str);
	}
	else
		cnt = 0;
	fclose (cnt_file);
	return cnt;
}

int     handler_init(void)
{
	sigemptyset (&save_sig);
	sigaddset (&save_sig, SIGUSR1);
	sigaddset (&save_sig, SIGUSR2);
	sigaddset (&save_sig, SIGINT);
	sigaddset (&save_sig, SIGPIPE);
	sigaddset (&save_sig, SIGHUP);
	sigaddset (&save_sig, SIGTERM);

	if (signal(SIGUSR1, handler) == SIG_ERR)
	{
		syslog (LOG_ERR, "signal (SIGUSR1) error");
		return FALSE;
	}	
        if (signal(SIGUSR2, handler) == SIG_ERR)
        {
                syslog (LOG_ERR, "signal (SIGUSR2) error");
                return FALSE;
        }
        if (signal(SIGINT, handler) == SIG_ERR)
        {
		syslog (LOG_ERR, "signal (SIGINT) error");
                return FALSE;
        }
        if (signal(SIGHUP, handler) == SIG_ERR)
        {
                syslog (LOG_ERR, "signal (SIGEPIPE) error");
                return FALSE;
        }
        if (signal(SIGQUIT, handler) == SIG_ERR)
        {
                syslog (LOG_ERR, "signal (SIGQUIT) error");
                return FALSE;
        }
	if (signal(SIGPIPE, handler) == SIG_ERR)
	{
		syslog (LOG_ERR, "signal (SIGPIPE) error");
		return FALSE;
	}
        if (signal(SIGTERM, handler) == SIG_ERR)
        {
                syslog (LOG_ERR, "signal (SIGTERM) error");
                return FALSE;
        }
        return TRUE;
}

int	send_dest_keep_alive(void)
{
	int k;
	int n;
	char	buff[80];
	struct  hostent *host;

        in_addr.sin_family = AF_INET;
        in_addr.sin_port = htons(dest_inet_port);
        in_addr.sin_addr.s_addr = inet_addr(dest_address);
        if (in_addr.sin_addr.s_addr == 0xffffffff)
        {
                host = gethostbyname(dest_address);
                if (host == NULL) return FALSE;
                in_addr.sin_addr.s_addr = *(unsigned int *)host->h_addr_list[0];
        }
	memset (buff, 0x00, sizeof(buff));
	memcpy (&buff[20], connect_call, 8);	
        sprintf (&buff[4], "%s", inet_ntoa (in_addr.sin_addr));
        sendto(in_addr_sock, &buff[4], 24, MSG_DONTWAIT, 
		(struct sockaddr *)&in_addr, sizeof(in_addr));
	time(&hole_punch_send_time);
	return TRUE;
}

void	send_keep_alive(void)
{
	int k;
	int n;
	char	buff[80];
	struct  hostent *host;

        in_addr.sin_family = AF_INET;
        in_addr.sin_port = htons(dest_inet_port);
        in_addr.sin_addr.s_addr = inet_addr(dest_address);
        if (in_addr.sin_addr.s_addr == 0xffffffff)
        {
                host = gethostbyname(dest_address);
                if (host == NULL) return;
                in_addr.sin_addr.s_addr = *(unsigned int *)host->h_addr_list[0];
        }
	memset (buff, 0x00, sizeof(buff));
	memcpy (&buff[20], connect_call, 8);	
        sprintf (&buff[4], "%s", inet_ntoa (in_addr.sin_addr));
        sendto(in_addr_sock, &buff[4], 24, MSG_DONTWAIT, 
		(struct sockaddr *)&in_addr, sizeof(in_addr));
	time(&hole_punch_send_time);

	memcpy (&buff[20], "dmoni", 5);
	memcpy (&buff[25], PACKAGE_VERSION, 5);
        time (&send_time);

	memcpy (&buff[32], MD5_dmonitor, 32);
	for (n = 0 ; n < 8 ; n++)
	{
		for (k = 0 ; k < 4 ; k++)
		{
			buff[32+(n*4)+k] ^= (send_time >> (k*8)) & 0xff;
		}
	}

	in_addr.sin_family = AF_INET;
	in_addr.sin_port = htons(hole_punch_port);
	in_addr.sin_addr.s_addr = inet_addr(hole_punch_server);
	if (in_addr.sin_addr.s_addr == 0xffffffff)
	{
		host = gethostbyname(hole_punch_server);
		if (host == NULL) return;
		in_addr.sin_addr.s_addr = *(unsigned int *)host->h_addr_list[0];
	}
	memcpy (buff, "HPCH", 4);
	memcpy (&buff[64], area_call, 8); 
	memcpy (&buff[72], zone_call, 7);
	buff[79] = 0x20;
	sendto(in_addr_sock, buff, 80, MSG_DONTWAIT, 
		(struct sockaddr *)&in_addr, sizeof(in_addr));
}

void	inet_send_buff_set(void)
{
	memset (inet_send_buff, 0x00, 58);
	memcpy (inet_send_buff, "DSTR", 4);

	inet_send_buff[4] = (m_seq >> 8) & 0xff;
	inet_send_buff[5] = m_seq & 0xff;

	inet_send_buff[6] = 's';
	inet_send_buff[7] = 0x12;
	//inet_send_buff[7] |= ZR;
	inet_send_buff[8] = 0x00;
	inet_send_buff[9] = 0x30;
	inet_send_buff[10] = 0x20;
	inet_send_buff[11] = 0x00;
	inet_send_buff[12] = 0x01;
	inet_send_buff[13] = 0x02;
	frameID = rand();
	inet_send_buff[14] = (frameID >> 8) & 0xff;
	inet_send_buff[15] = frameID & 0xff;
}

void	header_store (char header[])
{
	memcpy (header_save, header, 42);
}

int	send_rig (void)
{
	int	length;
	int	rate;

	length = getFifo (&send_buff[3]);
	if (length == 0)
	{
		memset (inet_frame_id, 0x00, 2);
		rig_send_sw = FALSE;
		return FALSE;
	}
	else if (length == 42)
	{
		rig_write (42, &send_buff[3]);
		seq = 0;
	}
	else if (length == 13)
	{
		send_buff[0] = 0x10;
		send_buff[1] = 0x22;
		send_buff[2] = seq & 0xff;
		if ((seq % 5) == 0) rig_led_onoff();
		send_buff[16] = 0xff;
		rig_write (17, send_buff);
		if (send_buff[3] & 0x40)
		{
			#ifdef	_DEBUG
			syslog (LOG_INFO, "Fifo_cnt (last frame) %d", Fifo_cnt);
			#endif
			if (Fifo_cnt < fifo_hold) 
			{
				rig_send_sw = FALSE;
				fifo_hold_limit = fifo_hold + 10;
			}
			else	fifo_hold_limit = Fifo_cnt + 10;
		}
	}
	return TRUE;
}

void    LastFrameSend (void)
{
	int	rate;

	#ifdef _DEBUG
	syslog(LOG_INFO, "LastFrameSend");
	#endif
        send_buff[0] = 0x10;
        send_buff[1] = 0x22;
        send_buff[2] = seq & 0xff;
	seq++;
	seq &= 0xff;
	recv_frame_seq++;
	recv_frame_seq %= 0x15;
	send_buff[3] = recv_frame_seq;
	memcpy (&send_buff[4], dummy_frame_1, 12);
        send_buff[16] = 0xff;
        rig_write (17, send_buff);
	
	if (recv_frame_seq % 2)
	{
		send_buff[2] = seq & 0xff;
		recv_frame_seq++;
		recv_frame_seq %= 0x15;
		send_buff[3] = recv_frame_seq;
		memcpy (&send_buff[4], dummy_frame_2, 12);
		rig_write (17, send_buff);
	}

        send_buff[2] = seq & 0xff;
        recv_frame_seq++;
        recv_frame_seq %= 0x15;
        send_buff[3] = recv_frame_seq;
        send_buff[3] |= 0x40;
        memcpy (&send_buff[4], dummy_frame_3, 12);
        rig_write (17, send_buff);
	drop_rate_print();
	inet_read_cnt = 0;
	inet_read_drop_cnt = 0;
	rig_last_frame_send = TRUE;
	if (Fifo_cnt < fifo_hold) rig_send_sw = FALSE;
}

int	getFifo	(char pkt[])
{
	struct	FifoPkt	*tmp;
	int	len;

	if (Rp->next ==	NULL) return 0;
	tmp = Rp;
	Rp = Rp->next;
	len = Rp->length;
	memcpy (pkt, Rp->pkt, Rp->length);
	free (tmp);
	Fifo_cnt--;
	#ifdef	_DEBUG
	syslog (LOG_INFO, "Fifo counter(getFifo):%d", Fifo_cnt);
	#endif
	return	len;
}

void    putFifo (int len, unsigned char pkt[])
{
        struct FifoPkt  *ret;

	#ifdef	_DEBUG
	syslog (LOG_INFO, "putFifo len:%d %2.2x", len, pkt[0]);
	#endif
        ret = malloc (sizeof(struct FifoPkt) - 1024 + len);
        if (ret == NULL)
        {
                syslog (LOG_ERR, "memory error in gw");
                return;
        }
        ret->next = NULL;
        ret->length = len;
        memcpy (ret->pkt, pkt, len);
        Wp->next = ret;
        Wp = ret;
        Fifo_cnt++;
	#ifdef	_DEBUG
	syslog (LOG_INFO, "Fifo counter:%d %2.2x", Fifo_cnt, rig_send_sw);
	#endif
	rig_fifo_check();
}

void	delete_last_header (void)
{
	struct FifoPkt  *ret;

	if (Wp->length != 42) return;
	ret = Wp;
	Wp = ret->next;
	Fifo_cnt--;
	free (ret);
}

void	connected_table(char str[])
{
	struct	stat	sb;
	int	i;
	char	cmd[128];
	int	ret;

	if (stat ("/var/tmp/", &sb)) return;
	if ((sb.st_mode & S_IFMT) != S_IFDIR) return;
	if (!memcmp (&str[4], "START", 5))
	{
		if (connected_fd != NULL) fclose (connected_fd);
		connected_fd = fopen (CONNECTED_LIST_TEMP, "w");
		fprintf (connected_fd, "<!DOCTYPE HTML>\n");
		fprintf (connected_fd, "<html lang=\"ja\"><head>\n");
		fprintf (connected_fd, "<meta http-equiv=\"Pragma\" content=\"no-cache\">\n");
		fprintf (connected_fd, "<meta http-equiv=\"Cache-Control\" content=\"no-cache\">\n");
		fprintf (connected_fd, "<meta name=\"expires\" content=\"0\">\n");
		//fprintf (connected_fd, "<meta http-equiv=\"refresh\" content=\"1;URL=./connected_table.html\">\n");
		//fprintf (connected_fd, "<meta http-equiv=\"refresh\" content=\"1\">\n");
		fprintf (connected_fd, "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n");
       		fprintf (connected_fd, "<script type=\"text/javascript\">\n");
        	fprintf (connected_fd, "<!--\n");
		fprintf (connected_fd, "setTimeout (function() {location.reload() ;} , 5000) ;\n"); 
		fprintf (connected_fd, "// -->\n</script>\n");
		fprintf (connected_fd, "</head>\n");
		fprintf (connected_fd, "<body style=\"background-color:#ffe6e6;\">\n");
		//fprintf (connected_fd, "<center>%8.8s 接続ユーザ 一覧 (%s)<br>\n", &str[36], &str[10]);
		fprintf (connected_fd, "<center>%8.8s 接続ユーザ 一覧 <br>\n", &str[36]);
		fprintf (connected_fd, "<table border=\"1\" style=\"font-size:0.9em;\">\n");
	}
	else if (!memcmp(&str[4], "END", 3))
	{
		if (connected_fd)
		{
			fprintf (connected_fd, "</table></center></body></html>\n");
			fclose (connected_fd);
			sprintf (cmd, "/var/www/cgi-bin/file_check  %s %s", CONNECTED_LIST_TEMP, CONNECTED_LIST);
			ret = system (cmd);
			connected_fd = NULL;
		}
	}
	else
	{
		fprintf (connected_fd, "<tr>\n");
		for (i = 0 ; i < 5 ; i++)
		{
			if (str[i*8+4] != 0x00) fprintf (connected_fd, "<td>%8.8s</td>\n", &str[i*8+4]);
		}
		fprintf (connected_fd, "</tr>\n");
	}
}

void	connected_table_clean(void)
{
	int	ret;

	ret = system("sudo cp /var/www/html/connected_table.html.save /var/tmp/connected_table.html");
}

void	error_msg (char str[])
{
        struct  stat    sb;
        int     i;
        char    cmd[128];
        int     ret;
	FILE	*error_fd;

        if (stat ("/var/tmp/", &sb)) return;
        if ((sb.st_mode & S_IFMT) != S_IFDIR) return;
        error_fd = fopen (ERROR_TEMP, "w");
	fprintf (error_fd, "<!DOCTYPE HTML>\n");
        fprintf (error_fd, "<html><head>\n");
	fprintf (error_fd, "<meta http-equiv=\"Pragma\" content=\"no-cache\">\n");
	fprintf (error_fd, "<meta http-equiv=\"Cache-Control\" content=\"no-cache\">\n");
	fprintf (error_fd, "<meta name=\"expires\" content=\"0\">\n");
	//fprintf (error_fd, "<meta http-equiv=\"refresh\" content=\"2;URL=./error_msg.html\">\n");
	//fprintf (error_fd, "<meta http-equiv=\"refresh\" content=\"2\">\n");
        fprintf (error_fd, "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n");
        fprintf (error_fd, "<script type=\"text/javascript\">\n");
        fprintf (error_fd, "<!--\n");
        fprintf (error_fd, "setTimeout (function() {location.reload() ;} , 5000) ;\n");
        fprintf (error_fd, "// -->\n</script>\n");
        fprintf (error_fd, "</head>\n");
        fprintf (error_fd, "<body style=\"background-color:#ff8c00;\">\n");
        fprintf (error_fd, "<center>%s</center>", &str[5]);
	fprintf (error_fd, "</body></html>");
	fclose (error_fd);
	sprintf (cmd, "/var/www/cgi-bin/file_check %s %s", ERROR_TEMP, ERROR_MSG);
        ret = system (cmd);
}

void    called_msg (char mycall[], char urcall[8])
{
        struct  stat    sb;
        int     i;
        char    cmd[128];
        int     ret;
        FILE    *called_fd;

        if (stat ("/var/tmp/", &sb)) return;
        if ((sb.st_mode & S_IFMT) != S_IFDIR) return;
        called_fd = fopen (ERROR_TEMP, "w");
        fprintf (called_fd, "<!DOCTYPE HTML>\n");
        fprintf (called_fd, "<html><head>\n");
	fprintf (called_fd, "<meta http-equiv=\"Pragma\" content=\"no-cache\">\n");
	fprintf (called_fd, "<meta http-equiv=\"Cache-Control\" content=\"no-cache\">\n");
	fprintf (called_fd, "<meta name=\"expires\" content=\"0\">\n");
        //fprintf (called_fd, "<meta http-equiv=\"refresh\" content=\"2;URL=./error_msg.html\">\n");
	fprintf (called_fd, "<meta http-equiv=\"refresh\" content=\"2\">\n");
        fprintf (called_fd, "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n");
        fprintf (called_fd, "</head>\n");
        fprintf (called_fd, "<body style=\"background-color:#ff8c00;\">\n");
        fprintf (called_fd, "<center>%8.8s が %8.8s を呼んでいます</center>", mycall, urcall);
        fprintf (called_fd, "</body></html>");
        fclose (called_fd);
        sprintf (cmd, "/var/www/cgi-bin/file_check  %s %s", ERROR_TEMP, ERROR_MSG);
        ret = system (cmd);
}

void    drop_rate_msg (char str[])
{
        struct  stat    sb;
        int     i;
        char    cmd[128];
        int     ret;
        FILE    *called_fd;

        if (stat ("/var/tmp/", &sb)) return;
        if ((sb.st_mode & S_IFMT) != S_IFDIR) return;
        called_fd = fopen (ERROR_TEMP, "w");
        fprintf (called_fd, "<!DOCTYPE HTML>\n");
        fprintf (called_fd, "<html><head>\n");
        fprintf (called_fd, "<meta http-equiv=\"Pragma\" content=\"no-cache\">\n");
        fprintf (called_fd, "<meta http-equiv=\"Cache-Control\" content=\"no-cache\">\n");
	fprintf (called_fd, "<meta name=\"expires\" content=\"0\">\n");
        //fprintf (called_fd, "<meta http-equiv=\"refresh\" content=\"2;URL=./error_msg.html\">\n");
	fprintf (called_fd, "<meta http-equiv=\"refresh\" content=\"2\">\n");
        fprintf (called_fd, "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n");
        fprintf (called_fd, "</head>\n");
        fprintf (called_fd, "<body style=\"background-color:#ff8c00;\">\n");
        fprintf (called_fd, "<center>%s</center>", str);
        fprintf (called_fd, "</body></html>");
        fclose (called_fd);
        sprintf (cmd, "/var/www/cgi-bin/file_check  %s %s", ERROR_TEMP, ERROR_MSG);
        ret = system (cmd);
}

void    error_msg_clean(void)
{
        int     ret;

        ret = system("sudo cp /var/www/html/error_msg.html.save /var/tmp/error_msg.html");
}

void	dmonitor_close (void)
{
	char	buff[24];
	struct  hostent *host;

        in_addr.sin_family = AF_INET;
        in_addr.sin_port = htons(dest_inet_port);
        in_addr.sin_addr.s_addr = inet_addr(dest_address);
        if (in_addr.sin_addr.s_addr == 0xffffffff)
        {
                host = gethostbyname(dest_address);
                if (host == NULL) return;
                in_addr.sin_addr.s_addr = *(unsigned int *)host->h_addr_list[0];
        }
        memset (buff, 0x00, sizeof(buff));
	memcpy (buff, "DISCONNECT", 10); 
        sendto(in_addr_sock, buff, 24, MSG_DONTWAIT,
                (struct sockaddr *)&in_addr, sizeof(in_addr));
}

void	jitter_buff_init(void)
{
	int	n;

	jitter_rd_pnt = 0;
	jitter_wr_cnt = 0;

	for (n = 0 ; n < 21 ; n++)
	{
		jitter_buff[n][0] = 0xff;
		if (n == 0) memcpy (&jitter_buff[n][1], NullVoice, 12);
		else        memcpy (&jitter_buff[n][1], NullVoice1, 12);
	}
}

void	jitter_put (unsigned char buff[])
{
	jitter_wr_pnt = buff[0] & 0x1f;
	if (jitter_wr_pnt > 20) return;
	if (buff[0])
		memcpy (&jitter_buff[jitter_wr_pnt][0], buff, 13);
	else
		memcpy (&jitter_buff[jitter_wr_pnt][0], buff, 10);
	jitter_wr_cnt++;
	if (jitter_wr_cnt < 0) jitter_wr_cnt = 0;
	#ifdef	_DEBUG_JITTER
	syslog (LOG_INFO, "jitter put %d %d", jitter_wr_pnt, jitter_wr_cnt);
	#endif
	jitter_putFifo();
}

void	jitter_putFifo (void)
{
	int k, n;

	if ((jitter_buff[jitter_wr_pnt][0] != 0xff) &&  (jitter_buff[jitter_wr_pnt][0] & 0x40))
	{
		#ifdef	_DEBUG_JITTER
		syslog(LOG_DEBUG, "last freame in jitter");
		#endif
		while (jitter_wr_pnt != jitter_rd_pnt)
		{
			if (jitter_buff[jitter_rd_pnt][0] == 0xff) 
			{
				jitter_buff[jitter_rd_pnt][0] = jitter_rd_pnt;
				inet_read_drop_cnt++;
				syslog (LOG_INFO, "packet loss (drop) %2.2x", jitter_rd_pnt);
			}
			putFifo (13, &jitter_buff[jitter_rd_pnt][0]);
			jitter_buff[jitter_rd_pnt][0] = 0xff;
			if (jitter_rd_pnt == 0) memcpy (&jitter_buff[jitter_rd_pnt][1], NullVoice, 12);
			else        memcpy (&jitter_buff[jitter_rd_pnt][1], NullVoice1, 12);
			jitter_rd_pnt++;
			if (jitter_rd_pnt > 20) jitter_rd_pnt = 0;
		}
		putFifo (13, &jitter_buff[jitter_wr_pnt][0]);
		jitter_buff[jitter_wr_pnt][0] = 0xff;
		jitter_rd_pnt = 0;
		return;
	}			
	while (jitter_buff[jitter_rd_pnt][0] != 0xff)
	{
		putFifo (13, &jitter_buff[jitter_rd_pnt][0]);
		jitter_buff[jitter_rd_pnt][0] = 0xff;
		if (jitter_rd_pnt == 0) memcpy (&jitter_buff[jitter_rd_pnt][1], NullVoice, 12);
		else        memcpy (&jitter_buff[jitter_rd_pnt][1], NullVoice1, 12);
		jitter_rd_pnt++;
		if (jitter_rd_pnt > 20) jitter_rd_pnt = 0;
		jitter_wr_cnt--;
	}
	#ifdef	_DEBUG_JITTER
	syslog (LOG_INFO, "debug %d %d", jitter_wr_pnt, jitter_rd_pnt);
	syslog (LOG_INFO, "%2.2x %2.2x %2.2x %2.2x %2.2x %2.2x %2.2x %2.2x %2.2x %2.2x %2.2x %2.2x %2.2x %2.2x %2.2x %2.2x %2.2x %2.2x %2.2x %2.2x %2.2x",
		jitter_buff[0][0], jitter_buff[1][0],jitter_buff[2][0],jitter_buff[3][0],jitter_buff[4][0],
		jitter_buff[5][0],jitter_buff[6][0],jitter_buff[7][0],jitter_buff[8][0],jitter_buff[8][0],jitter_buff[10][0],
		jitter_buff[11][0], jitter_buff[12][0],jitter_buff[13][0],jitter_buff[14][0],jitter_buff[15][0],
		jitter_buff[16][0],jitter_buff[17][0],jitter_buff[18][0],jitter_buff[19][0],jitter_buff[20][0]);
	#endif
	n = 0;
	if (((jitter_wr_pnt + 1) != jitter_rd_pnt) && ((jitter_wr_pnt + 1) != (jitter_rd_pnt + 21)))
	{
		k = jitter_wr_pnt;
		while (k != jitter_rd_pnt)
		{
			if (jitter_buff[k][0] != 0xff)
			{
				n++;
			}
			else
				break;
			k--;
			if (k < 0) k = 20;
		}
	#ifdef	_DEBUG_JITTER
	syslog (LOG_INFO, "n:%d\n", n);
	#endif
	}
	if (n > 4)
	{
		while (jitter_rd_pnt != jitter_wr_pnt)
		{
			if (jitter_buff[jitter_rd_pnt][0] == 0xff) inet_read_drop_cnt++;
			jitter_buff[jitter_rd_pnt][0] = jitter_rd_pnt;
			putFifo (13, &jitter_buff[jitter_rd_pnt][0]);
			jitter_buff[jitter_rd_pnt][0] = 0xff;
			if (jitter_rd_pnt == 0) memcpy (&jitter_buff[jitter_rd_pnt][1], NullVoice, 12);
			else        memcpy (&jitter_buff[jitter_rd_pnt][1], NullVoice1, 12);
			jitter_rd_pnt++;
			if (jitter_rd_pnt > 20) jitter_rd_pnt = 0;
		}
		jitter_wr_cnt = 0;
	}
}

void	drop_rate_print (void)
{
	int	rate;
	char	str[64];

        if (inet_read_cnt)
        {
                rate = (inet_read_drop_cnt * 10000) / inet_read_cnt;
                syslog(LOG_INFO, "drop packet rate %d.%02d%% (%ld/%ld)",
                        rate/100, rate%100, inet_read_drop_cnt, inet_read_cnt);
                syslog(LOG_INFO, "jitter info. ave:%ldmSec. max:%ldmSec. min:%ldmSec.",
                        jitter_ave/inet_read_cnt + 20, jitter_max, jitter_min);
		memset (str, 0x00, sizeof(str));
		sprintf (str, "drop packet rate %d,%02d%% (%ld/%ld)", 
			rate/100, rate%100, inet_read_drop_cnt, inet_read_cnt);
		drop_rate_msg (str);
        }
}

int    ETagGen (char file_name[], char md_string[])
{
        MD5_CTX ctx;
        unsigned char   md[MD5_LBLOCK];
        FILE    *fp;
        int     ret;
	char	temp[512];

        fp = fopen (file_name, "rb");
	if (!fp) return -1;
        MD5_Init (&ctx);
        while (1)
        {
                ret = fread (temp, 1, sizeof(temp), fp);
                if (ret > 0) MD5_Update(&ctx, temp, ret);
                else break;
        }
        fclose (fp);
        MD5_Final (md, &ctx);
        sprintf (md_string, "%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x",
                        md[0], md[1], md[2], md[3], md[4], md[5], md[6], md[7],
                        md[8], md[9], md[10], md[11], md[12], md[13], md[14], md[15]);
	return 0;
}
